%% 2D Periodic Graphene Structure in CheckerBoard Pattern
%% Spec : Square Lattice, rectangular unit cell
%% It uses Approximate Boundary Condition

% clc
clear all;
q=sqrt(-1);

%% Constants
epsilon0=1e-9/36/pi;                    % Vaccum permittivity
c=3e8;                                  % light velocity in vaccum (m/s)
El = 1.6021757e-19;                     % electron charge (coulomb)
hb = 6.582e-16;                         % Plank Constant bar (h/2/pi) (eV.s)
kB = 8.6173324e-5;                      % The Boltzman cinstant (eV/K)
%% 

FF1=[1:0.5:69.5 70:0.2:129.8 130:0.5:300];

Counter1=0;
Re=zeros(length(FF1),1);
Tr=zeros(length(FF1),1);
Summasion=zeros(length(FF1),1);
PE = hb*2*pi*c./(FF1*1e-6); 

for Counter1=1:length(FF1)       
   
lambda=FF1(Counter1)*1e-6; % free space wavelength

w=2*pi*c./lambda;   % the angular frequency
d1=8e-6;            % Period of the Structure in x direction 
d2=8e-6;            % Period of the Structure in y direction 

h3=0.5e-9;          % Delta h = 0.5 - 2 - 5 nm 

x1=[-d1/2 0 d1/2];
x2=[-d2/2 0 d2/2];

%% the permittivity, Permeability and Wave Number in Upper and Lower Layers

epsilonL=3;        % The relative Permeativity of the Last layer of the structure (i.e. the Upest layer) 
nL=sqrt(epsilonL);  % The Refractive index of the Last layer of the structure 
muL=1;              % The relative permeability of the Last layer of the structure (i.e. the Upest layer) 


epsilon1=4;        % The relative permeativity of the First layer of the structure (i.e. the lowest layer) 
n1=sqrt(epsilon1);  % The Refractive index of the First layer of the structure  
mu1=1;              % The relativepermeability of the First layer of the structure (i.e. the Upest layer) 

k0=2*pi./lambda;
kL=k0*nL;
k1=k0*n1;

%% % the permittivity & Permeability of Periodic Layer

Ef = 0.6;                               % Fermi Level (eV)
Tau = 0.25e-12;                         % Relaxation Time (s)
    
Sigmaa = conductivity(Ef*El,Tau,lambda);  % the Conductivity of the Graphene in the periodic layer in SI units

Sigmaa = Sigmaa /4/pi/epsilon0;         % the Conductivity of the Graphene in the periodic layer in Gaussian units
Sigmaa = Sigmaa*4*pi/c;                 % Modification of Conductivity
Sigmab = 0;                             % the Conductivity of background region in the periodic layer 

%% Effective CONDUVTIVITY

Sigmae=-q*k0*h3*(epsilon1+epsilonL)/2; % Excess Conducticvity
Sigmaa = Sigmaa + Sigmae;
Sigmab = Sigmab + Sigmae;

%% % The Incident Angles and Incident Wave Vector Components

teta = (0.001)*pi/6;      % Normal incident (Note that when teta=0 and n4=1, two arrays of C4 will be NaN)
phi  = (0)*pi/6;
zeta = (0)*pi/6;          %!!!!! in parallelogram unit cell it should be 30 degree.

%% The main loop parameters

Ms=20;
Mf=20;
FF2=Ms:Mf;

Counter2=0;


TH1=0;       % Thereshold value for eigenvalue selection
TH2=1e-5;


for Counter2=1:length(FF2)
    
    M=FF2(Counter2); % The number of Fourier Coeff. is (2*M+1) in x1 direction
    N=M;             % The number of Fourier Coeff. is (2*N+1) in x2 direction
    
    if length(FF1)==1
        CounterT=Counter2;
        FF=FF2;
    else
        CounterT=Counter1;
        FF=FF1;
    end
    CounterT;
    t=(2*M+1)*(2*N+1); % t is used to define the dimension of Matrices
    
    
%% Calculation of wave vector components of different orders in x1 direction (they are the same for all layers)  
    alfa0=kL*sin(teta)*cos(phi);
    alfamm=zeros(t,1);
    alfamm(1:t)=alfa0+(2*pi/d1).*kron((-M:M),ones(1,2*N+1)); 
    alfa=diag(alfamm);
    alfa=sparse(alfa);
    
    % Calculation of wave vector components of different orders in x2 direction (they are the same for all layers) 
    beta0=kL*sin(teta)*sin(phi+zeta);
    betann=zeros(t,1);
    betann(1:t)=beta0+(2*pi/d2).*kron(ones(1,2*M+1),(-N:N));
    beta=diag(betann);
    beta=sparse(beta);
    
%% Calculation of propagation constants and W matrices of the first layer by Rayleigh expansion
    
    Gama1o=zeros(t,1);
    Gama1o(1:t)=sqrt(k1^2-(sec(zeta))^2*(alfamm.^2+betann.^2-2*sin(zeta)*alfamm.*betann));
    Gama1t=zeros(2*t,1);
    Gama1t(1:2:2*t)=Gama1o;
    Gama1t(2:2:2*t)=-Gama1o;
    k=0;
    Gama1p=zeros(t,1);
    for i=1:2*t
        
        if abs(imag(Gama1t(i)))<TH2*abs(real(Gama1t(i)))
            
            if real(Gama1t(i))>TH1
                k=k+1;
                Gama1p(k)=Gama1t(i);
            end
            
        else
             
            if imag(Gama1t(i))>TH1
                k=k+1;
                Gama1p(k)=Gama1t(i);
            end
            
        end
        
    end
    
    A1=zeros(t,1);
    A1(1:t)=k0*epsilon1*sec(zeta)./Gama1p-alfamm.^2./(k0*mu1*cos(zeta)*Gama1p);
    A1=diag(A1);
   
    B1=zeros(t,1);
    B1(1:t)=k0*epsilon1*sec(zeta)./Gama1p-betann.^2./(k0*mu1*cos(zeta)*Gama1p);
    B1=diag(B1);
 
    F1=zeros(t,1);
    F1(1:t)=k0*epsilon1*tan(zeta)./Gama1p-alfamm.*betann./(k0*mu1*cos(zeta)*Gama1p);
    F1=diag(F1);
    
    Exp1=zeros(t,1);
    Exp1(1:t)=exp(q*Gama1p*h3/2);
    Exp1I=zeros(t,1);
    Exp1I=1./Exp1;
    Exp1=diag(Exp1);
    Exp1I=diag(Exp1I);


%% Calculation of propagation constants and W matrices of the Last layer by Rayleigh expansion
    
    
    if nL==n1
        
        GamaLp=Gama1p;
        AL=A1;
        BL=B1;
        FL=F1;
        ExpL=Exp1;
        ExpLI=Exp1I;
    else
       
        GamaLo=zeros(t,1);
        GamaLo(1:t)=sqrt(kL^2-(sec(zeta))^2*(alfamm.^2+betann.^2-2*sin(zeta)*alfamm.*betann));
        GamaLt=zeros(2*t,1);
        GamaLt(1:2:2*t)=GamaLo;
        GamaLt(2:2:2*t)=-GamaLo;
        k=0;
        GamaLp=zeros(t,1);
        for i=1:2*t
        
            if abs(imag(GamaLt(i)))<TH2*abs(real(GamaLt(i)))
            
                if real(GamaLt(i))>TH1
                    k=k+1;
                    GamaLp(k)=GamaLt(i);
                end
            
            else
             
                if imag(GamaLt(i))>TH1
                    k=k+1;
                    GamaLp(k)=GamaLt(i);
                end
            
            end
        
        end
        

        AL=zeros(t,1);
        AL(1:t)=k0*epsilonL*sec(zeta)./GamaLp-alfamm.^2./(k0*muL*cos(zeta)*GamaLp);
        AL=diag(AL);
   
        BL=zeros(t,1);
        BL(1:t)=k0*epsilonL*sec(zeta)./GamaLp-betann.^2./(k0*muL*cos(zeta)*GamaLp);
        BL=diag(BL);
 
        FL=zeros(t,1);
        FL(1:t)=k0*epsilonL*tan(zeta)./GamaLp-alfamm.*betann./(k0*muL*cos(zeta)*GamaLp);
        FL=diag(FL);
        
        ExpL=zeros(t,1);
        ExpL(1:t)=exp(q*GamaLp*h3/2);
        ExpLI=zeros(t,1);
        ExpLI=1./ExpL;
        ExpL=diag(ExpL);
        ExpLI=diag(ExpLI);

    end
    
%% Making the Main S Matrices 

    % Inverse rule and then Laurrent rule
    % Sigma12 : inverse rule in x1 direction and then Laurent rule in x2 direction
    % Sigma21 : inverse rule in x2 direction and then Laurent rule in x1 direction     
    
    % Note that LI2DCHChekerboard1D function is the same with LI2DCHChekerboard. It's not necessary to have many harmonics in x2 direction
    % for 1D structure. So for 1D structures M~=N and N is at least. IN
    % LI2DCHChekerboard1D function A1 and A2  matrices are made twice.
    
    [Sigma12 Sigma21]=LI2DCHChekerboard(x1,x2,d1,d2,Sigmaa,Sigmab,M,N);    
    
    SigmaI=FC2DCHChekerboard(x1,x2,d1,d2,1/Sigmaa,1/Sigmab,M,N);
    INVSigmaI=inv(SigmaI);
    clear SigmaI
    
    X1 = [+BL.*ExpL+cos(zeta)*Sigma12+tan(zeta)*sin(zeta)*INVSigmaI ...
                  -FL.*ExpL-tan(zeta)*INVSigmaI;...
                  +FL.*ExpL+tan(zeta)*INVSigmaI ...
                  -AL.*ExpL-cos(zeta)*Sigma21-tan(zeta)*sin(zeta)*INVSigmaI];
    Y2 = [-B1.*Exp1 F1.*Exp1;-F1.*Exp1 A1.*Exp1];

    
    Y1 = [B1.*Exp1I -F1.*Exp1I;F1.*Exp1I -A1.*Exp1I];
    X2 = [-BL.*ExpLI+cos(zeta)*Sigma12+tan(zeta)*sin(zeta)*INVSigmaI ...
                  +FL.*ExpLI-tan(zeta)*INVSigmaI;...
                  -FL.*ExpLI+tan(zeta)*INVSigmaI ...
                  +AL.*ExpLI-cos(zeta)*Sigma21-tan(zeta)*sin(zeta)*INVSigmaI];
    
    INV  = inv(X1-Y2);
    R3ud = INV*(Y2-X2);
    T3dd = INV*(X1-X2);

    clear X1 X2 Y1 Y2 INV  B1 A1 F1 BL AL FL INVSigmaI Sigma12 Sigma21 Exp1 Exp1I ExpL ExpLI
    
%% Incident field in the fourth region


    I1=sqrt(GamaLp(M*(2*N+1)+N+1)/(kL^2-beta0^2)); %(E2(Ey)=0 & E1(Ex)=~0 TM Polarization)
    I2=0;


    % incident field in the fourth region
    dL=zeros(2*t,1);
    dL(M*(2*N+1)+N+1)=I1;
    dL(t+M*(2*N+1)+N+1)=I2;   
    Z1=R3ud*dL;
    clear R3ud;
    uL1=Z1(1:t);
    uL2=Z1(t+1:2*t);
    clear Z1;
    Z1=T3dd*dL;
    clear T3dd;
    d11=Z1(1:t);
    d12=Z1(t+1:2*t);
    clear Z1;


   %%  Diffraction efficiencies
    
    % Calculation of the Reflected Diffraction Efficiency of different orders in
    % fourth region
    DER=zeros(t,1);
    DER(1:t)=real((1./(GamaLp)).*((kL^2-alfamm.^2).*(abs(uL2)).^2+...
                                 (kL^2-betann.^2).*(abs(uL1)).^2+...
                                 (alfamm.*betann-kL^2*sin(zeta)).*(uL1.*conj(uL2)+uL2.*conj(uL1))));
    
    % Calculation of the Transmitted Diffraction Efficiency of different orders in
    % first region                         
    DET=zeros(t,1);
    DET(1:t)=real((1./(Gama1p)).*((k1^2-alfamm.^2).*(abs(d12)).^2+...
                                 (k1^2-betann.^2).*(abs(d11)).^2+...
                                 (alfamm.*betann-k1^2*sin(zeta)).*(d11.*conj(d12)+d12.*conj(d11))));
                             
    Re(CounterT)=sum(DER);                          % Reflection
    Tr(CounterT)=sum(DET);                          % Transmission

    % energy consevation ( for lossless material it should be 1 and in the case of lossy materials its less than 1)
    Summasion(CounterT)=sum(DER)+sum(DET);                  


    clear Gama1p GamaLp alfamm betann dL uL1 uL2;
    clear d11;
    clear d12;


end

end


hold on
plot(FF,1-Re-Tr,'.-k')