%% 2D Periodic Graphene Structure in CheckerBoard Pattern
%% Spec : Square Lattice, rectangular unit cell
%% It model Graphene disks as a region with complex permittivity

% clc
clear all;
q=sqrt(-1);

X0=0.17585810159583864509258432917704 ; % the converged result for relative error calculation

%% Constants
epsilon0=1e-9/36/pi;                    % Vaccum permittivity
c=3e8;                                  % light velocity in vaccum (m/s)
El = 1.6021757e-19;                     % electron charge (coulomb)
hb = 6.582e-16;                         % Plank Constant bar (h/2/pi) (eV.s)
kB = 8.6173324e-5;                      % The Boltzman cinstant (eV/K)
%%  

FF1=92.3;  % the Resonance wavelegth when up and down epsilon are 3 and 4 respectively
Counter1=0;

PE = hb*2*pi*c./(FF1*1e-6); 

for Counter1=1:length(FF1)       
   
lambda=FF1(Counter1)*1e-6; % free space wavelength

w=2*pi*c./lambda;   % the angular frequency
d1=8e-6;            % Period of the Structure in x direction 
d2=8e-6;            % Period of the Structure in y direction 

h3=0.5e-9;          % The thickness of the Periodic layer 

x1=[-d1/2 0 d1/2];
x2=[-d2/2 0 d2/2];

%% the permittivity, Permeability and Wave Number in Upper and Lower Layers

epsilonL=3;         % The relative Permeativity of the Last layer of the structure 
nL=sqrt(epsilonL);  % The Refractive index of the Last layer of the structure 
muL=1;              % The relative permeability of the Last layer of the structure 


epsilon1=4;        % The relative permeativity of the First layer of the structure 
n1=sqrt(epsilon1);  % The Refractive index of the First layer of the structure  
mu1=1;              % The relativepermeability of the First layer of the structure 

k0=2*pi./lambda;
kL=k0*nL;
k1=k0*n1;

%% % the permittivity & Permeability of Periodic Layer

Ef = 0.6;                               % Fermi Level (eV)
Tau = 0.25e-12;                         % Relaxation Time (s)

Sigmaa = conductivity(Ef*El,Tau,lambda); % the Conductivity of the Graphene in the periodic layer in SI units

Sigmaa = Sigmaa /4/pi/epsilon0;         % the Conductivity of the Graphene in the periodic layer in Gaussian units    

epsilona=1+q*(4*pi/c)*Sigmaa./k0/h3;     % the relative permittivity of the material that makes cylinders in the periodic layer in Gaussian Units

epsilonb=1;                      % the relative permittivity of background region in the periodic layer 
mu=1;                            % the relative permeability of the periodic layer (in this formulation mua=mub=mu) 

%% % The Incident Angles and Incident Wave Vector Componets

teta = (0.001)*pi/6;      % Normal incident (Note that when teta=0 and n4=1, two arrays of C4 will be NaN)
phi  = (0)*pi/6;
zeta = (0)*pi/6;      %!!!!! in parallelogram unit cell it should be 30 degree.

%% The main loop parameters

Ms=8;
Mf=15;
FF2=Ms:Mf;

Re=zeros(length(FF2),1);
Tr=zeros(length(FF2),1);
Summasion=zeros(length(FF2),1);
Counter2=0;


TH1=0;       % Thereshold value for eigenvalue selection
TH2=1e-5;


for Counter2=1:length(FF2)
    
    M=FF2(Counter2); % The number of Fourier Coeff. is (2*M+1) in x1 direction
    N=M;             % The number of Fourier Coeff. is (2*N+1) in x2 direction
    
    if length(FF1)==1
        CounterT=Counter2;
        FF=FF2;
    else
        CounterT=Counter1;
        FF=FF1;
    end
    CounterT;
    t=(2*M+1)*(2*N+1); % t is used to define the dimension of Matrices
    
%% Calculation of wave vector components of different orders in x1 direction (they are the same for all layers)  
    alfa0=kL*sin(teta)*cos(phi);
    alfamm=zeros(t,1);
    alfamm(1:t)=alfa0+(2*pi/d1).*kron((-M:M),ones(1,2*N+1)); 
    alfa=diag(alfamm);
    alfa=sparse(alfa);
    
    % Calculation of wave vector components of different orders in x2 direction (they are the same for all layers) 
    beta0=kL*sin(teta)*sin(phi+zeta);
    betann=zeros(t,1);
    betann(1:t)=beta0+(2*pi/d2).*kron(ones(1,2*M+1),(-N:N));
    beta=diag(betann);
    beta=sparse(beta);


%% Calculation of propagation constants and W matrices of the first layer by Rayleigh expansion
    
    Gama1o=zeros(t,1);
    Gama1o(1:t)=sqrt(k1^2-(sec(zeta))^2*(alfamm.^2+betann.^2-2*sin(zeta)*alfamm.*betann));
    Gama1t=zeros(2*t,1);
    Gama1t(1:2:2*t)=Gama1o;
    Gama1t(2:2:2*t)=-Gama1o;
    k=0;
    Gama1p=zeros(t,1);
    for i=1:2*t
        
        if abs(imag(Gama1t(i)))<TH2*abs(real(Gama1t(i)))
            
            if real(Gama1t(i))>TH1
                k=k+1;
                Gama1p(k)=Gama1t(i);
            end
            
        else
             
            if imag(Gama1t(i))>TH1
                k=k+1;
                Gama1p(k)=Gama1t(i);
            end
            
        end
        
    end
    
    C1=zeros(t,1);
    C1(1:t)=(alfamm.*betann-k1^2*sin(zeta))./(mu1*k0*cos(zeta)*Gama1p);
    C1=diag(C1);
   
    A1=zeros(t,1);
    A1(1:t)=-(k1^2-alfamm.^2)./(mu1*k0*cos(zeta)*Gama1p);
    A1=diag(A1);

    B1=zeros(t,1);
    B1(1:t)=(k1^2-betann.^2)./(mu1*k0*cos(zeta)*Gama1p);
    B1=diag(B1);

    W11=eye(2*t);
    W11=sparse(W11);
    W12=[-C1,A1;B1,C1];
    W12=sparse(W12);

    clear A1 B1 C1


%% Making the Main Matrices F & G, in the Periodic Layer

    % 2D Fourier Coefficients of (epsilon)
    Mcoeff=FC2DCHChekerboard(x1,x2,d1,d2,epsilona,epsilonb,M,N);

    % F (Eq. (33) of Li, 1997 article)
    F11=zeros(t);
    F12=zeros(t);
    F21=zeros(t);
    F22=zeros(t);
    F11=-mu*k0^2*sin(zeta)*eye(t)+alfa*inv(Mcoeff)*beta;
    F12=+mu*k0^2*eye(t)-alfa*inv(Mcoeff)*alfa;
    F21=-mu*k0^2*eye(t)+beta*inv(Mcoeff)*beta;
    F22=+mu*k0^2*sin(zeta)*eye(t)-beta*inv(Mcoeff)*alfa;
    F=[F11 F12;F21 F22];
    clear F11 F12 F21 F22 Mcoeff
    
    % 2D Fourier Coefficients of (1/epsilon)
    Micoeff=FC2DCHChekerboard(x1,x2,d1,d2,1/epsilona,1/epsilonb,M,N);
    INVMicoeff=inv(Micoeff);
    clear Micoeff
    
    % Inverse rule and then Laurrent rule
    
    % Note that LI2DCHChekerboard1D function is the same with LI2DCHChekerboard. 
    % It's not necessary to have many harmonics in x2 direction
    % for 1D structure. So for 1D structures M~=N and N is at least. IN
    % LI2DCHChekerboard1D function A1 and A2  matrices are made twice.
    
    [Coeffb Coeffbb]=LI2DCHChekerboard(x1,x2,d1,d2,epsilona,epsilonb,M,N);

    % G  (Eq. (34) of Li, 1997 article)
    G11=zeros(t);
    G12=zeros(t);
    G21=zeros(t);
    G22=zeros(t);
    G11=-alfa*beta+mu*k0^2*sin(zeta)*INVMicoeff;
    G12=+alfa*alfa-(mu*k0^2)*((cos(zeta))^2*(Coeffbb)+(sin(zeta))^2*INVMicoeff);
    G21=-beta*beta+(mu*k0^2)*((cos(zeta))^2*(Coeffb)+(sin(zeta))^2*INVMicoeff);
    G22=alfa*beta-mu*k0^2*sin(zeta)*INVMicoeff;
    G=[G11 G12;G21 G22];

    clear G11 G12 G21 G22 Coeffb Coeffbb INVMicoeff Mcoeff Micoeff alfa beta;

    % Eigenvalue and Eigenvector 
    [W31 EVo]=eig(F*G);
    clear F;
    EVo=diag(EVo);   
    EVo=(EVo.^.5)/(sqrt(mu)*k0*cos(zeta));

    % EV is the eigenvalues i.e. propagation constants of different orders in z direction.
    EV = zeros(4*t,1);
    EV (1:2:4*t) = EVo;
    EV (2:2:4*t) = -EVo;
    clear EVo
    EVp = zeros(2*t,1); % Eigen values with positive imaginary parts
    EVn = zeros(2*t,1); % Eigen values with negative imaginary parts
    j=0;
    k=0;
        
    % If the imaginary part of eigen value is very smaller than the real part, it is removed and eigen value is real; 
     for i=1:4*t
         if abs(imag(EV(i)))<TH2*abs(real(EV(i)))
             EV(i)=real(EV(i));
         end
     end
        
     for i=1:4*t

         if imag(EV(i))<TH1
             j=j+1;
             EVn(j)=EV(i);
         end
         if imag(EV(i))>TH1
             k=k+1;
             EVp(k)=EV(i);
         end
         if imag(EV(i))==TH1
             if (real(EV(i)))<TH1
                 j=j+1;
                 EVn(j)=EV(i);
             else
                 k=k+1;
                 EVp(k)=EV(i);
             end
         end
            
     end

     IEVp=zeros(2*t,1);
     IEVp(1:2*t)=EVp(1:2*t).^(-1);
     IEVp=diag(IEVp);
     W32=(sec(zeta)/mu/k0)*(G*W31)*(IEVp);

     clear IEVp G EVn;

%% Calculation of propagation constants and W matrices of the Last layer by Rayleigh expansion
    
    
    if nL==n1
        GamaLp=Gama1p;
        WL1=W11;
        WL2=W12;
    else
        GamaLo=zeros(t,1);
        GamaLo(1:t)=sqrt(kL^2-(sec(zeta))^2*(alfamm.^2+betann.^2-2*sin(zeta)*alfamm.*betann));
        GamaLt=zeros(2*t,1);
        GamaLt(1:2:2*t)=GamaLo;
        GamaLt(2:2:2*t)=-GamaLo;
        k=0;
        GamaLp=zeros(t,1);
        for i=1:2*t
        
            if abs(imag(GamaLt(i)))<TH2*abs(real(GamaLt(i)))
            
                if real(GamaLt(i))>TH1
                    k=k+1;
                    GamaLp(k)=GamaLt(i);
                end
            
            else
             
                if imag(GamaLt(i))>TH1
                    k=k+1;
                    GamaLp(k)=GamaLt(i);
                end
            
            end
        
        end
        

        CL=zeros(t,1);
        CL(1:t)=(alfamm(1:t).*betann-kL^2*sin(zeta))./(muL*k0*cos(zeta)*GamaLp);
        CL=diag(CL);

        AL=zeros(t,1);
        AL(1:t)=-(kL^2-alfamm.^2)./(muL*k0*cos(zeta)*GamaLp);
        AL=diag(AL);

        BL=zeros(t,1);
        BL(1:t)=(kL^2-betann.^2)./(muL*k0*cos(zeta)*GamaLp);
        BL=diag(BL);

        WL1=eye(2*t);
        WL1=sparse(WL1);
        WL2=[-CL,AL;BL,CL];
        
    end
    
    
    clear CL BL AL ;

%% S Matrix Algorithm (the  W-S scheme) (with symmetry)

    Q11=inv(W31)*W11;
    Q12=inv(W32)*W12;
    clear W11 W12;
    ta1=inv(Q11+Q12);
    clear Q11;
    R1ud=eye(2*t)-2*Q12*ta1;
    clear Q12;
    T1dd=2*ta1;
    clear ta1;

    Q31=inv(WL1)*W31;
    clear WL1 W31;
    Q32=inv(WL2)*W32;
    clear WL2 W32;

    Omega3P=zeros(2*t,1);
    Omega3P(1:2*t)=exp(q*h3.*EVp(1:2*t));
    clear EVp;
    Omega3P=diag(Omega3P);
    Omega3P=sparse(Omega3P);

    Rt1ud=Omega3P*R1ud*Omega3P;
    clear R1ud;
    F3=Q31*(eye(2*t)+Rt1ud);
    clear Q31;
    G3=Q32*(eye(2*t)-Rt1ud);
    clear Rt1ud Q32;
    ta3=inv(F3+G3);
    clear F3;
    R3ud=eye(2*t)-2*G3*ta3;
    clear G3;
    Tt1dd=T1dd*Omega3P;
    clear Omega3P T1dd;
    T3dd=2*Tt1dd*ta3;
    clear ta3 Tt1dd;

%% Incident field in the fourth region

    I1=sqrt(GamaLp(M*(2*N+1)+N+1)/(kL^2-beta0^2)); %(E2(Ey)=0 & E1(Ex)=~0 TM Polarization)
    I2=0;


    % incident field in the fourth region
    dL=zeros(2*t,1);
    dL(M*(2*N+1)+N+1)=I1;
    dL(t+M*(2*N+1)+N+1)=I2;   
    Z1=R3ud*dL;
    clear R3ud;
    uL1=Z1(1:t);
    uL2=Z1(t+1:2*t);
    clear Z1;
    Z1=T3dd*dL;
    clear T3dd;
    d11=Z1(1:t);
    d12=Z1(t+1:2*t);
    clear Z1;


   %%  Diffraction efficiencies
    
    % Calculation of the Reflected Diffraction Efficiency of different orders in
    % fourth region
    DER=zeros(t,1);
    DER(1:t)=real((1./(GamaLp)).*((kL^2-alfamm.^2).*(abs(uL2)).^2+...
                                 (kL^2-betann.^2).*(abs(uL1)).^2+...
                                 (alfamm.*betann-kL^2*sin(zeta)).*(uL1.*conj(uL2)+uL2.*conj(uL1))));
    
    % Calculation of the Transmitted Diffraction Efficiency of different orders in
    % first region                         
    DET=zeros(t,1);
    DET(1:t)=real((1./(Gama1p)).*((k1^2-alfamm.^2).*(abs(d12)).^2+...
                                 (k1^2-betann.^2).*(abs(d11)).^2+...
                                 (alfamm.*betann-k1^2*sin(zeta)).*(d11.*conj(d12)+d12.*conj(d11))));
                             
    Re(CounterT)=sum(DER);                          % Reflection
    Tr(CounterT)=sum(DET);                          % Transmission
    
    % energy consevation ( for lossless material it should be 1 and in the case of lossy materials its less than 1)
    Summasion(CounterT)=sum(DER)+sum(DET);        

    clear Gama1p GamaLp alfamm betann dL uL1 uL2;
    clear d11;
    clear d12;



end

end

hold on
plot(FF,abs(((1-Re-Tr)-X0)/X0),'.r')